% Matlab script to construct any additional sbatch arguments for job
% being submitted
% Part of UMD Matlab-Slurm integration scripts
% 
% This is an attempt to make a generic version which can be called
% from both shared and remote versions of configCluster to do most of
% the work.
%
% Author: Tom Payerle (payerle@umd.edu) 
% (c) University of Maryland 2022-2023
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% This file is part of the git repo 
% git@gitlab.umd.edu:acigs-enduser-sw/umd-slurm-matlab-scripts.git
% Please make any and all modifications there
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

function commonSubmitArgs = getCommonSubmitArgs(cluster)
% Get any additional submit arguments for the Slurm sbatch command
% that are common to both independent and communicating jobs.
%
% Uses validatedPropValue and validatedPropValueWithAlias functions
%
% Copyright 2022-2023 University of Maryland
%

    commonSubmitArgs = '';
    ap = cluster.AdditionalProperties;

    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    %%                    UMD Customized fields                       %%
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    % AccountName: name of Slurm allocation account to charge to
    %        (aliases Account)
    % ??? ClusterName ???: not sure how to handle/what to do
    % EnableDebug: logical.  If set, this script will print debug messages
    % EmailAddress: used to set --mail-user flag.  Ignored unless
    %    EmailType is also set.  Takes an email address (e.g. 'me@gmail.com')
    % EmailType: used to set --mail-type flag.  Takes a string of
    %    job email states (comma delimitted, no spaces).  Eg. 'BEGIN,END'
    % Features: used to set features in --constraint flag
    %    Should be a comma separated list of features (no spaces)
    %    E.g. 'rhel8,amd'
    %    (aliases Feature, Constraint)
    % GpuCard: sets type of GPU card to request when GpusPerNode is set.
    %    Ignored if GpusPerNode is set.  If omitted, just set quantity
    %    not type of GPU (probably not what is desired)
    %    E.g. 'a100' or 'a00_1g.5gb'
    %    (aliases GpuType)
    % GpusPerNode: Specifies number of GPUs per node needed.  If set and
    %    non-zero, will add a --gpus-per-node flag requesting this number
    %    of GPUs of type GpuCard (or this number of any GPUs if GpuCard
    %    not set).  If set and non-zero, will also *default* Partition
    %    to GPU.  Should be a non-negative integer if set.
    % MemUsage: Sets the amount of memory per CPU core to request. If set,
    %    will be passed as argument to --mem-per-cpu flag.  Should be
    %    a positive integer (unit defaults to MB), or a positive
    %    integer followed by M or G for MB or GB.
    %    (aliases MemPerCpu)
    % NumNodes: If set, will be passed as the value to the --nodes flag.
    %    Expect a positive integer.  Normally should be omitted and
    %    let scheduler choose the number of nodes
    %    (aliases Nodes)
    % ProcsPerNode: If set, will be passed as the value to the --ntasks-per-node
    %    sbatch flag, which will set the maximum number of tasks (i.e. MATLAB
    %    workers) which will be assigned to a single node.  Normally should
    %    be omitted and let the scheduler choose.
    % Partition: If set, it will be passed as the value to the --partition
    %    sbatch flag.  If omitted and GpusPerNode not set or zero, no
    %    --partition flag will be passed (i.e. allow Slurm to default it).
    %    If omitted and GpusPerNode is set and non-zero, will default to 'gpu'.
    %    (aliases QueueName)
    % RequiresExclusiveMode: logical.  If set, will pass the --exclusive flag
    %    to sbatch.  
    % Reservation: If set, will be passed as the value to the --reservation
    %    sbatch flag, causing job to run in the named reservation.
    % Resources: If set, will be passed as the value to the --gres sbatch flag.
    %    (aliases Resource)
    % WallTime: If set, will be passed as the value to the --time sbatch flag.
    %
    %
    % NOTE re Aliases:  These are expanded using the function
    %    validatedPropValueWithAliases, which will search for a property with
    %    the primary name, then alias names (in order given), then deprecated
    %    alias names (in order given).
    %    The first match found will be returned (using validatedPropValue
    %    function).  If no match found, a default value will be returned.
    %    If the first match is in the deprecated list, a deprecation warning
    %    will be displayed.
    %    If multiple names match, only the first will be used and warnings will
    %    be generated for each duplicate match.
    %    If EnableDebug is set, matches from the alias list will generate a
    %    debug message.

    debugMode = validatedPropValue(ap, 'EnableDebug', 'logical', logical(0) );

    % AccountName
    pname = "AccountName";
    ptype = 'string';
    defaultValue = [];
    aliases = [ "Account" ];
    deprecated = [];
    tmp = validatedPropValueWithAliases( ...
       ap, pname, ptype, defaultValue, aliases, deprecated, debugMode);
    if ~isempty(tmp)
       commonSubmitArgs = strtrim(sprintf( ...
           '%s --account=%s', commonSubmitArgs, tmp));
    end

    % EmailAddress 
    tmp = validatedPropValue(ap, "EmailAddress", 'string');
    if ~isempty(tmp)
       commonSubmitArgs = strtrim(sprintf( ...
           '%s --mail-user=%s', commonSubmitArgs, tmp));
    end

    % EmailType
    tmp = validatedPropValue(ap, "EmailType", 'string');
    if ~isempty(tmp)
        commonSubmitArgs = strtrim(sprintf( ...
            '%s --mail-type=%s', commonSubmitArgs, tmp));
    end

    % Features
    pname = "Features";
    ptype = 'string';
    defaultValue = [];
    aliases = [ "Feature", "Constraint" ];
    deprecated = [];
    tmp = validatedPropValueWithAliases( ...
       ap, pname, ptype, defaultValue, aliases, deprecated, debugMode);
    if ~isempty(tmp)
       commonSubmitArgs = strtrim(sprintf( ...
           '%s --constraint=%s', commonSubmitArgs, tmp));
    end

    % GpuCard and  GpusPerNode
    pname = "GpuCard";
    ptype = 'string';
    defaultValue = [];
    aliases = [ "GpuType" ];
    deprecated = [];
    gpuCard = validatedPropValueWithAliases( ...
       ap, pname, ptype, defaultValue, aliases, deprecated, debugMode);

    pname = "GpusPerNode";
    ptype = 'numeric';
    defaultValue = 0;
    gpusPerNode = validatedPropValue(ap, pname,ptype, defaultValue);
    defaultPartition = [];

    if gpusPerNode > 0
        % Requested GPUs
        if isempty(gpuCard)
            % GPU count given, but no type
            gpuStr = strtrim(sprintf( '%s', gpusPerNode));
            if debugMode
                fprintf(2, '[DEBUG] WARNING: GpusPerNode set w/out GpuCard');
            end
        else
            gpuStr = strtrim(sprintf( '%s:%s', gpuCard, gpusPerNode));
        end

        commonSubmitArgs = strtrim(sprintf( ...
           '%s --gpus-per-node=%s', commonSubmitArgs, gpuStr));
        % And set default Partition to 'gpu'
        defaultPartition = 'gpu';
    else
        if ~isempty(gpuCard)
            if debugMode
                fprintf(2, ...
		    '[DEBUG] WARNING: GpuCard ignored when GpusPerNode=0');
            end
        end
    end

    % MemUsage
    pname = "MemUsage";
    ptype = 'string';
    defaultValue = [];
    aliases = [ "MemPerCpu" ];
    deprecated = [];
    tmp = validatedPropValueWithAliases( ...
       ap, pname, ptype, defaultValue, aliases, deprecated, debugMode);
    if ~isempty(tmp)
       commonSubmitArgs = strtrim(sprintf( ...
           '%s --mem-per-cpu=%s', commonSubmitArgs, tmp));
    end

    % NumNodes
    pname = "NumNodes";
    ptype = 'numeric';
    defaultValue = [];
    aliases = [ "Nodes" ];
    deprecated = [];
    tmp = validatedPropValueWithAliases( ...
       ap, pname, ptype, defaultValue, aliases, deprecated, debugMode);
    if ~isempty(tmp)
       commonSubmitArgs = strtrim(sprintf( ...
           '%s --nodes=%s', commonSubmitArgs, tmp));
    end

    % ProcsPerNode
    pname = "ProcsPerNode";
    ptype = 'numeric';
    defaultValue = [];
    aliases = [ ];
    deprecated = [];
    tmp = validatedPropValue( ap, pname, ptype, defaultValue);
    if ~isempty(tmp)
       commonSubmitArgs = strtrim(sprintf( ...
           '%s --ntasks-per-node=%s', commonSubmitArgs, tmp));
    end

    % Partition
    pname = "Partition";
    ptype = 'string';
    defaultValue = defaultPartition;
    aliases = [ "QueueName" ];
    deprecated = [];
    tmp = validatedPropValueWithAliases( ...
       ap, pname, ptype, defaultValue, aliases, deprecated, debugMode);
    if ~isempty(tmp)
       commonSubmitArgs = strtrim(sprintf( ...
           '%s --partition=%s', commonSubmitArgs, tmp));
       % Warn if defaultPartition was set and Partition set to something else
       % e.g. if using GPUs and overrode Partition
       if ~isempty(defaultPartition) && tmp ~= defaultPartition
           if debugMode
               fprintf(2, '[DEBUG] WARNING: using Partition=%s instead ' + ...
                   'of recommended default of %s', tmp, defaultPartition);
           end
       end
    end

    % RequiresExclusiveMode
    pname = "RequiresExclusiveMode";
    ptype = 'logical';
    defaultValue = [];
    aliases = [];
    deprecated = [];
    tmp = validatedPropValue( ap, pname, ptype, defaultValue);
    if ~isempty(tmp)
       if tmp
           % RequiresExclusiveMode is set
           commonSubmitArgs = strtrim(sprintf( ...
               '%s --exclusive', commonSubmitArgs));
       else
           % RequiresExclusiveMode is explicitly unset
           commonSubmitArgs = strtrim(sprintf( ...
               '%s --oversubscribe', commonSubmitArgs));
       end
    end

    % Reservation
    pname = "Reservation";
    ptype = 'string';
    defaultValue = [];
    aliases = [];
    deprecated = [];
    tmp = validatedPropValue( ap, pname, ptype, defaultValue);
    if ~isempty(tmp)
       commonSubmitArgs = strtrim(sprintf( ...
           '%s --reservation=%s', commonSubmitArgs, tmp));
    end

    % Resources
    pname = "Resources";
    ptype = 'string';
    defaultValue = [];
    aliases = [];
    deprecated = [];
    tmp = validatedPropValue( ap, pname, ptype, defaultValue);
    if ~isempty(tmp)
       commonSubmitArgs = strtrim(sprintf( ...
           '%s --gres=%s', commonSubmitArgs, tmp));
    end

    % WallTime
    pname = "WallTime";
    ptype = 'string';
    defaultValue = [];
    aliases = [];
    deprecated = [];
    tmp = validatedPropValue( ap, pname, ptype, defaultValue);
    if ~isempty(tmp)
       commonSubmitArgs = strtrim(sprintf( ...
           '%s --time=%s', commonSubmitArgs, tmp));
    end

    % Catch-all for any sbatch parameters not handled above
    % Append any arguments provided by the AdditionalSubmitArgs field of cluster.AdditionalProperties.
    extraArgs = validatedPropValue(ap, 'AdditionalSubmitArgs', 'char');
    if ~isempty(extraArgs)
        commonSubmitArgs = strtrim(sprintf('%s %s', commonSubmitArgs, extraArgs));
    end

    if debugMode
        fprintf(1, '[DEBUG] commonSubmitArgs set to "%s"\n', commonSubmitArgs);
    end
end
