% Matlab script to return a structure array containing data on UMD HPC clusters
% Part of UMD Matlab-Slurm integration scripts
% 
% Author: Tom Payerle (payerle@umd.edu) 
% (c) University of Maryland 2022-2023
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% This file is part of the git repo 
% git@gitlab.umd.edu:acigs-enduser-sw/umd-slurm-matlab-scripts.git
% Please make any and all modifications there
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

function info = UmdHpcClusterInfo(clusterName, allowBadCluster)
% Returns a structure array containing information regarding MATLAB on the
% MD HPC cluster specified by clusterName
%
% clusterName should be the name of an UMD HPC cluster,e.g. zaratan
% allowBadCluster is a logical
%
% On success, we return a structure array with fields
%	clusterName: name of cluster, all lowercase
%	ClusterName: name of cluster, in normal case
%	loginNode: name of login node for cluster
%	matlabVersions: array of structure arrays, one for each MATLAB
%		version supported on the cluster.  The version structures
%		have fields:
%			version: MATLAB version, e.g. 2022b
%			path: path to root of installation
%	remoteJobStorageLocation: string representing the default location
%		for remoteJobStorageLocation on this cluster.  It should
%		contain one "%s" which will be replaced with users login name
% If no cluster is found with clusterName = clusterName, a fatal error is
% raised UNLESS allowBadCluster is set, in which case we return []
%
% All Errors are fatal.
%
    narginchk(1,2);
    if nargin < 2
        allowBadCluster = logical(0);
    end

    % Master cell array with information about all clusters
    % We just have to return the matching element
    % NOTE: Use double quotes for string values, especially clusterName
    cluster_info_list = [ ...
        % Zaratan cluster ...
        struct( ...
            "clusterName", "zaratan", ...
            "ClusterName", "Zaratan", ...
            "loginNode", "login.zaratan.umd.edu", ...
            "matlabVersions",  ...
            [    struct( ...
                    "version", "2022b", ...
                    "path", "/software/matlab/2022b/Linux" ...
                 ), ...
            ], ...
            "remoteJobStorageLocation", ...
                "/home/%s/scratch/MatlabParallelServer" ...
        ),
        % Juggernaut cluster ...
        struct( ...
            "clusterName", "juggernaut", ...
            "ClusterName", "Juggernaut", ...
            "loginNode", "login.juggernaut.umd.edu", ...
            "matlabVersions",  ...
            [    struct( ...
                    "version", "2021b", ...
                    "path", "/software/matlab/2021b/Linux" ...
                 ), ...
                 struct( ...
                    "version", "2021a", ...
                    "path", "/software/matlab/2021a/Linux" ...
                 ), ...
                 struct( ...
                    "version", "2020b", ...
                    "path", "/software/matlab/2020b/Linux" ...
                 ), ...
            ], ...
            "remoteJobStorageLocation", ...
                "/lustre/jn10/%s/MatlabParallelServer" ...
        ),
    ];

    % Convert clusterName to all lowercase (for matching)
    clusterName = lower(clusterName);

    numClusters = length(cluster_info_list);
    info = [];
    known_cluster_names = [];
    for index = 1:numClusters
        cinfo = cluster_info_list(index);
        cname = cinfo.clusterName;
        known_cluster_names = [ known_cluster_names, cname ];
        if cname == clusterName
            info = cinfo;
            break;
        end
    end

    if isempty(info)
        if ~allowBadCluster
            tmp = strjoin(known_cluster_names, ', ');
            error('UMD:GenericSlurm:UnknownCluster', ...
               '[ERROR] Unknown cluster "%s"; expecting one of [%s]\n', ...
               clusterName, tmp);
        end
    end
end
